import 'dart:io';

import 'package:analyzer/dart/analysis/analysis_context.dart';

import 'package:analyzer/dart/analysis/results.dart';
import 'package:analyzer/dart/analysis/session.dart';
import 'package:analyzer/dart/ast/ast.dart';
import 'package:_fe_analyzer_shared/src/parser/parser.dart';
import 'package:_fe_analyzer_shared/src/scanner/recover.dart';
import 'package:_fe_analyzer_shared/src/scanner/token.dart';
import 'package:report/parser/write_token.dart';
import 'property_path.dart';
import 'vist_node.dart';
import 'package:report/parser/report.dart';

class DebugListener extends Listener {}

Future<void> analyzeSingleFile(AnalysisContext context, String path, ReportData reportData) async {
  AnalysisSession session = context.currentSession;
  DateTime begin = DateTime.now();
  ResolvedUnitResult resultA = await session.getResolvedUnit(path);
  print(' == 耗时 ${session.runtimeType} ${DateTime.now().difference(begin)}');
  CompilationUnit unit = resultA.unit; //AST 的根结点

  for (final CompilationUnitMember unitMember in unit.declarations) {
    if (unitMember is ClassDeclaration) {
      ConstructorVisit constructorVisit = ConstructorVisit();
      unitMember.visitChildren(constructorVisit);
      if (constructorVisit.list.isNotEmpty) {
        Parser parser = Parser(DebugListener());
        constructorVisit.list.forEach((element) async {
          Property property = element.property;
          Map<String, String> map = reportData.reportData(property?.uuidPosition?.endToken?.lexeme);
          if (map != null && map.containsKey('name')) {
            if (property.namePosition != null) {
              Token token = synthesizeToken(property.namePosition.endToken.offset, '\'${map['name']}\'', TokenType.STRING);
              parser.rewriter.replaceTokenFollowing(property.namePosition.endToken.previous, token);
            } else {
              /// name : ***
              Token commaToken = SyntheticStringToken(TokenType.COMMA, ',', property.uuidPosition.endToken.offset, 0);
              commaToken = parser.rewriter.insertToken(property.uuidPosition.endToken, commaToken);

              Token nameToken = synthesizeToken(commaToken.offset, 'name', TokenType.STRING);
              nameToken = parser.rewriter.insertToken(commaToken, nameToken);

              Token colonToken = synthesizeToken(nameToken.offset, ':', TokenType.COLON);
              colonToken = parser.rewriter.insertToken(nameToken, colonToken);

              Token newToken = synthesizeToken(property.uuidPosition.endToken.offset, '\'${map['name']}\'', TokenType.STRING);
              parser.rewriter.insertToken(colonToken, newToken);
            }
            if (element.constructorBegin != null && map.containsKey('comment')) {
              if (element.constructorBegin is SimpleToken) {
                SimpleToken token = element.constructorBegin as SimpleToken;
                String string = token?.precedingComments?.lexeme ?? '';
                Token commentToken =
                    CommentToken(TokenType.SINGLE_LINE_COMMENT, '${string} \/\/todo ${map['comment']} ', element.constructorBegin.previous.offset);
                token.precedingComments = commentToken;
              }
            }
          }
        });

        // printToken(unit.beginToken);
        writeToken(unit.beginToken, path, unit.lineInfo);
      }
    }
  }
}

void printToken(Token token) {
  while (!token.isEof) {
    print('${token.lexeme} ');
    if (token.precedingComments != null) {
      print('\n ${token.precedingComments?.lexeme ?? ""} ');
      Token nextPrecedingComment = token.precedingComments?.next;
      while (nextPrecedingComment != null) {
        print('\n ${nextPrecedingComment?.lexeme ?? ""} ');
        nextPrecedingComment = nextPrecedingComment.next;
      }
    }
    token = token.next;
  }
}
